﻿(function (self: ServiceWorkerGlobalScope) {

    //template:begin
    //this section is going to be replaced...
    const version = "2/15/2019 1:00:00 PM";
    const assetList = [
        '/file1.js',
        '/file2.aspx'    
    ];
    //template:end

    self.addEventListener("install", function (event: ExtendableEvent) {
        console.log('WORKER: install event in progress.');
        event.waitUntil(
            /* The caches built-in is a promise-based API that helps you cache responses,
               as well as finding and deleting them.
            */
            caches
                /* You can open a cache by name, and this method returns a promise. We use
                   a versioned cache name here so that we can remove old cache entries in
                   one fell swoop later, when phasing out an older service worker.
                */
                .open(version)
                .then(function (cache) {
                    /* After the cache is opened, we can fill it with the offline fundamentals.
                       The method below will add all resources we've indicated to the cache,
                       after making HTTP requests for each of them.
                    */
                    var requests = assetList.map((asset) => {
                        return cache.add(asset).catch(err => {
                            //we use this instead of addAll so we can see what is failing
                            console.error("Could not download " + asset);
                        });
                    });

                    return Promise.all(requests);
                })
                .then(function () {
                    console.log('WORKER: install completed');
                })
        );
    });

    self.addEventListener("fetch", function (event: FetchEvent) {
        console.log('WORKER: fetch event in progress.');

        // if the request has query parameters, `hasQuery` will be set to `true`
        var hasQuery = event.request.url.indexOf('?') != -1;

        /* We should only cache GET requests, and deal with the rest of method in the
           client-side, by handling failed POST,PUT,PATCH,etc. requests.
        */
        if (event.request.method !== 'GET') {
            /* If we don't block the event as shown below, then the request will go to
               the network as usual.
            */
            console.log('WORKER: fetch event ignored.', event.request.method, event.request.url);
            return;
        }

        // if the request has query parameters, `hasQuery` will be set to `true`
        var hasQuery = event.request.url.indexOf('?') != -1;

        /* Similar to event.waitUntil in that it blocks the fetch event on a promise.
           Fulfillment result will be used as the response, and rejection will end in a
           HTTP response indicating failure.
        */
        event.respondWith(
            caches
                /* This method returns a promise that resolves to a cache entry matching
                   the request. Once the promise is settled, we can then provide a response
                   to the fetch request.
                */
                .match(event.request, {
                    ignoreSearch: hasQuery, //this is a known bug it is extremely slow, we know that things that are static are not used
                    ignoreVary: true,
                    ignoreMethod: true
                })
                .then(function (cached) {
                    if (cached) {
                        console.log(event.request.url + " Found");
                    } else {
                        console.log(event.request.url + " Not Found");
                    }

                    return cached || fetch(event.request);
                })
        );
    });


    self.addEventListener("activate", function (event: ExtendableEvent) {

        /* Just like with the install event, event.waitUntil blocks activate on a promise.
           Activation will fail unless the promise is fulfilled.
        */
        console.log('WORKER: activate event in progress.');

        event.waitUntil(
            caches
                /* This method returns a promise which will resolve to an array of available
                   cache keys.
                */
                .keys()
                .then(function (keys) {
                    // We return a promise that settles when all outdated caches are deleted.
                    return Promise.all(
                        keys
                            .filter(function (key) {
                                // Filter by keys that don't start with the latest version prefix.
                                return !key.startsWith(version);
                            })
                            .map(function (key) {
                                /* Return a promise that's fulfilled
                                   when each outdated cache is deleted.
                                */
                                console.log('WORKER: deleting key ' + key);
                                return caches.delete(key);
                            })
                    );
                })
                .then(function () {
                    console.log('WORKER: activate completed.');
                })
        );
    });

    self.addEventListener('message', function (event: MessageEvent) {
        console.log('WORKER: message recieved.');
        console.log(event.data);
        switch (event.data) {
            case "claimController":
                console.log('WORKER: claimController.');
                self.clients.claim().catch(err => {
                    console.log(err);
                });
                break;
            case "skipWaiting":
                console.log('WORKER: skipWaiting.');
                self.skipWaiting().catch(err => {
                    console.log(err);
                });
                break;
            default:
                throw event.data + "is not a valid message for the appCacheServiceWorker."
        }
    });
})(<ServiceWorkerGlobalScope>self);