var fs = require('fs');
var cheerio = require('cheerio');
var through = require('through2');
var gulp = require('gulp');
var gutil = require('gulp-util');
var replace = require('gulp-string-replace');
var PluginError = gutil.PluginError;


// Consts
var PLUGIN_NAME = 'gulp-service-worker-app-cache-builder';

function buildAppCache(options) {
    var excludeString = options.ExcludeString;
    var projectFile = options.ProjectFile;
    var serviceWorkerFile = options.ServiceWorkerFile;

    var excludes = excludeString.split(",");
    var contentNames = [];
    var newLine = "\n";

    var fileContents = fs.readFileSync(serviceWorkerFile, "utf8");

    return through.obj(function (file, encoding, done) {

        var $ = cheerio.load(file.contents);

        $('Content').each(function (index, contentNode) {

            var fileName = "/" + contentNode.attribs["include"].replace(/\\/g, "\/");
            var exclude = false;

            for (var i = 0; i < excludes.length; i++) {
                var ext = excludes[i].toLowerCase();
                if (ext.startsWith(".")) {
                    if (fileName.toLowerCase().endsWith(ext)) {
                        exclude = true;
                        break;
                    }
                }
                else {
                    if (fileName.toLowerCase().includes(ext)) {
                        exclude = true;
                        break;
                    }
                }
            }
            if (!exclude) {
                contentNames.push(fileName);
            }
        });

        var output = newLine;//make sure we start on a new line

        output += "const version = '" + new Date().toString() + "';" + newLine;

        output += "const assetList = [" + newLine;
        //add the cache section
        contentNames.forEach(function (name, index) {
            let nameString = "'" + name + "'";
            if (index === 0) {
                output += nameString;
            } else {
                output += "," + newLine + nameString;
            }
        });

        output += newLine + "];" + newLine;

        var re = new RegExp("(\/\/template:begin)(.*?)(\/\/template:end)", 's');
        var matches = fileContents.match(re);

        if (matches === null || matches.length!==4) {
            throw "The options.ServiceWorkerFile[" + serviceWorkerFile + "] does not contain a replaceable area demarcated by \\\\template:begin...\\\\template:end";
        }

        //replace the content portion of the regex
        fileContents = fileContents.replace(re, "$1" + output + "$3");

        //add the new contents to the buffer for output
        file.contents = Buffer(fileContents);

        //push the file to the output stream
        this.push(file);
        done();
    });
}


// Plugin level function(dealing with files)
function appCacheBuilder(options) {

    if (!options) {
        throw new PluginError(PLUGIN_NAME, 'options required!');
    }
    if (!options || !options.ProjectFile || !fs.existsSync(options.ProjectFile)) {
        throw new PluginError(PLUGIN_NAME, 'options.ProjectFile must be a valid csproj file.');
    }
    if (!options || !options.ServiceWorkerFile || !fs.existsSync(options.ServiceWorkerFile)) {
        throw new PluginError(PLUGIN_NAME, 'options.ServiceWorkerFile must be an existing file.');
    }

    return gulp
        .src(options.ProjectFile)
        .pipe(buildAppCache(options));
}

// Exporting the plugin main function
module.exports = appCacheBuilder;
